==> lebiniou_3.15.orig.tar.gz: lebiniou-3.15/plugins/stable/output/erlang/v7.c <==
/*
 *  Copyright 1994-2012 Olivier Girondel
 *
 *  This file is part of lebiniou.
 *
 *  lebiniou is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  lebiniou is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with lebiniou. If not, see <http://www.gnu.org/licenses/>.
 */

#include "erlang.h"

/*
 * Protocol v7:
 *
 * Input:
 * Output:
 *  [C][To, Cmd, Arg]        // keyboard command
 *  [U][C|P|S][uint32_t id]  // update event
 */

extern uint32_t last_colormap;
extern uint32_t last_picture;
extern uint32_t last_sequence;

extern char send_colormap_update;
extern char send_picture_update;
extern char send_sequence_update;


static void
v7_send_event(const Event_t *e)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  u_char val;
  int ret;

  /* send packet size */
  total = sizeof(uint32_t)+4*sizeof(u_char);
  total2 = htonl(total);
  ret = write_exact((const u_char *)&total2, sizeof(uint32_t));

  /* protocol version */
  proto2 = htonl(proto);
  ret = write_exact((const u_char *)&proto2, sizeof(uint32_t));

  val = CMD_CHAR;
  ret = write_exact(&val, sizeof(u_char));
  val = e->to;
  ret = write_exact(&val, sizeof(u_char));
  val = e->cmd;
  ret = write_exact(&val, sizeof(u_char));
  val = e->arg0;
  ret = write_exact(&val, sizeof(u_char));

  /* XXX keep gcc happy */
  ret++; /* <- remove this ! */
}


static void
v7_send_events(Context_t *ctx)
{
  GList *t;

  for (t = ctx->events; t != NULL; t = g_list_next(t)) {
    Event_t *e = (Event_t *)t->data;
    v7_send_event(e);
  }
}


static void
v7_send_id(const u_char what, const uint32_t id)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  uint32_t id2;
  u_char val;
  int ret;

  /* send packet size */
  total = sizeof(uint32_t)+2*sizeof(u_char)+sizeof(uint32_t);
  total2 = htonl(total);
  ret = write_exact((const u_char *)&total2, sizeof(uint32_t));

  /* protocol version */
  proto2 = htonl(proto);
  ret = write_exact((const u_char *)&proto2, sizeof(uint32_t));

  val = UPDATE_CHAR;
  ret = write_exact(&val, sizeof(u_char));
  val = what;
  ret = write_exact(&val, sizeof(u_char));

  id2 = htonl(id);
  ret = write_exact((const u_char *)&id2, sizeof(uint32_t));

  /* XXX keep gcc happy */
  ret++; /* <- remove this ! */
}


static void
v7_send_sequence(Sequence_t *seq)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  uint32_t id, id2;
  u_char val;
  int ret;
  GList *tmp;
  uint8_t seq_size, i;
  
  seq_size = Sequence_size(seq);
  
  /* send packet size */
  total = sizeof(uint32_t)           /* proto */
    + 2*sizeof(u_char)               /* 'US' */
    + sizeof(uint32_t)               /* sequence id */
    + sizeof(uint32_t)               /* lens id or 0 */
    + sizeof(uint8_t)                /* sequence size */
    + (seq_size * (sizeof(uint32_t)+sizeof(u_char))); /* N plugins */ /* TODO plugin_id -> uint32_t */
  total2 = htonl(total);
  ret = write_exact((const u_char *)&total2, sizeof(uint32_t));

  /* protocol version */
  proto2 = htonl(proto);
  ret = write_exact((const u_char *)&proto2, sizeof(uint32_t));

  val = UPDATE_CHAR;
  ret = write_exact(&val, sizeof(u_char));
  val = UPDATE_SEQUENCE_CHAR;
  ret = write_exact(&val, sizeof(u_char));

  /* sequence id */
  id2 = htonl(seq->id);
  ret = write_exact((const u_char *)&id2, sizeof(uint32_t));

  /* the sequence */
  /*   - lens plugin id or 0 */
  id = (NULL != seq->lens) ? seq->lens->id : 0;
  id2 = htonl(id);
  ret = write_exact((const u_char *)&id2, sizeof(uint32_t));

  /*   - size of the sequence */
  ret = write_exact((const u_char *)&seq_size, sizeof(uint8_t));

  /*   - layers */
  tmp = seq->layers;
  for (i = 0; i < seq_size; i++) {
    Layer_t *l = (Layer_t *)tmp->data;

    /* plugin id */
    id2 = htonl(l->plugin->id);
    ret = write_exact((const u_char *)&id2, sizeof(uint32_t));

    /* layer mode */
    ret = write_exact((const u_char *)&l->mode, sizeof(u_char));

    tmp = g_list_next(tmp);
  }

  /* XXX keep gcc happy */
  ret++; /* <- remove this ! */
}


void
v7(Context_t *ctx)
{
  v7_send_events(ctx);

  if (send_sequence_update)
    if (ctx->sm->cur->id != last_sequence) {
      v7_send_sequence(ctx->sm->cur);
      last_sequence = ctx->sm->cur->id;
      send_colormap_update = send_picture_update = 0;
    }

  if (send_colormap_update)
    if (ctx->sm->cur->cmap_id != last_colormap) {
      v7_send_id(UPDATE_COLORMAP_CHAR, ctx->sm->cur->cmap_id);
      last_colormap = ctx->sm->cur->cmap_id;
    }

  if (send_picture_update)
    if (ctx->sm->cur->picture_id != last_picture) {
      v7_send_id(UPDATE_PICTURE_CHAR, ctx->sm->cur->picture_id);
      last_picture = ctx->sm->cur->picture_id;
    }
}

==> lebiniou_3.18.orig.tar.gz: lebiniou-3.18/plugins/stable/output/erlang/v7.c <==
/*
 *  Copyright 1994-2012 Olivier Girondel
 *
 *  This file is part of lebiniou.
 *
 *  lebiniou is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  lebiniou is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with lebiniou. If not, see <http://www.gnu.org/licenses/>.
 */

#include "erlang.h"

/*
 * Protocol v7:
 *
 * Input:
 * Output:
 *  [C][To, Cmd, Arg]        // keyboard command
 *  [U][C|P|S][uint32_t id]  // update event
 */

extern uint32_t last_colormap;
extern uint32_t last_picture;
extern uint32_t last_sequence;

extern char send_colormap_update;
extern char send_picture_update;
extern char send_sequence_update;


static void
v7_send_event(const Event_t *e)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  u_char val;

  /* send packet size */
  total = sizeof(uint32_t)+4*sizeof(u_char);
  total2 = htonl(total);
  write_uint32(&total2);

  /* protocol version */
  proto2 = htonl(proto);
  write_uint32(&proto2);

  /* command (to, cmd, arg0) */
  val = CMD_CHAR;
  write_uint8(&val);
  val = e->to;
  write_uint8(&val);
  val = e->cmd;
  write_uint8(&val);
  val = e->arg0;
  write_uint8(&val);
}


static void
v7_send_events(Context_t *ctx)
{
  GList *t;

  for (t = ctx->events; t != NULL; t = g_list_next(t)) {
    Event_t *e = (Event_t *)t->data;
    v7_send_event(e);
  }
}


static void
v7_send_id(const u_char what, const uint32_t id)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  uint32_t id2;
  u_char val;

  /* send packet size */
  total = sizeof(uint32_t)+2*sizeof(u_char)+sizeof(uint32_t);
  total2 = htonl(total);
  write_uint32(&total2);

  /* protocol version */
  proto2 = htonl(proto);
  write_uint32(&proto2);

  val = UPDATE_CHAR;
  write_uint8(&val);
  val = what;
  write_uint8(&val);

  id2 = htonl(id);
  write_uint32(&id2);
}


static void
v7_send_sequence(Sequence_t *seq)
{
  uint32_t proto = 7, proto2;
  uint32_t total, total2;
  uint32_t id, id2;
  u_char val;
  GList *tmp;
  uint8_t seq_size, i;
  
  seq_size = Sequence_size(seq);
  
  /* send packet size */
  total = sizeof(uint32_t)           /* proto */
    + 2*sizeof(u_char)               /* 'US' */
    + sizeof(uint32_t)               /* sequence id */
    + sizeof(uint32_t)               /* lens id or 0 */
    + sizeof(uint8_t)                /* sequence size */
    + (seq_size * (sizeof(uint32_t)+sizeof(u_char))); /* N plugins */ /* TODO plugin_id -> uint32_t */
  total2 = htonl(total);
  write_uint32(&total2);

  /* protocol version */
  proto2 = htonl(proto);
  write_uint32(&proto2);

  val = UPDATE_CHAR;
  write_uint8(&val);
  val = UPDATE_SEQUENCE_CHAR;
  write_uint8(&val);

  /* sequence id */
  id2 = htonl(seq->id);
  write_uint32(&id2);

  /* the sequence */
  /*   - lens plugin id or 0 */
  id = (NULL != seq->lens) ? seq->lens->id : 0;
  id2 = htonl(id);
  write_uint32(&id2);

  /*   - size of the sequence */
  write_uint8(&seq_size);

  /*   - layers */
  tmp = seq->layers;
  for (i = 0; i < seq_size; i++) {
    Layer_t *l = (Layer_t *)tmp->data;

    /* plugin id */
    id2 = htonl(l->plugin->id);
    write_uint32(&id2);

    /* layer mode */
    write_uint8((uint8_t *)&l->mode);

    tmp = g_list_next(tmp);
  }
}


void
v7(Context_t *ctx)
{
  v7_send_events(ctx);

  if (send_sequence_update)
    if (ctx->sm->cur->id != last_sequence) {
      v7_send_sequence(ctx->sm->cur);
      last_sequence = ctx->sm->cur->id;
      send_colormap_update = send_picture_update = 0;
    }

  if (send_colormap_update)
    if (ctx->sm->cur->cmap_id != last_colormap) {
      v7_send_id(UPDATE_COLORMAP_CHAR, ctx->sm->cur->cmap_id);
      last_colormap = ctx->sm->cur->cmap_id;
    }

  if (send_picture_update)
    if (ctx->sm->cur->picture_id != last_picture) {
      v7_send_id(UPDATE_PICTURE_CHAR, ctx->sm->cur->picture_id);
      last_picture = ctx->sm->cur->picture_id;
    }
}
